@@ -91,7 +91,24 @@ def find_resource(manager, name_or_id, **kwargs):
 
     """
 
-    # Try to get entity as integer id
+    # Case 1: name_or_id is an ID, we need to call get() directly
+    # for example: /projects/454ad1c743e24edcad846d1118837cac
+    # For some projects, the name only will work. For keystone, this is not
+    # enough information, and domain information is necessary.
+    try:
+        return manager.get(name_or_id)
+    except Exception:
+        pass
+
+    # Case 2: name_or_id is a name, but we have query args in kwargs
+    # for example: /projects/demo&domain_id=30524568d64447fbb3fa8b7891c10dd6
+    try:
+        return manager.get(name_or_id, **kwargs)
+    except Exception:
+        pass
+
+    # Case 3: Try to get entity as integer id. Keystone does not have integer
+    # IDs, they are UUIDs, but some things in nova do, like flavors.
     try:
         if isinstance(name_or_id, int) or name_or_id.isdigit():
             return manager.get(int(name_or_id), **kwargs)
@@ -101,17 +118,14 @@ def find_resource(manager, name_or_id, **kwargs):
     #                 of client exceptions.
     except Exception as ex:
         if (type(ex).__name__ == 'NotFound' or
-                type(ex).__name__ == 'HTTPNotFound'):
+                type(ex).__name__ == 'HTTPNotFound' or
+                type(ex).__name__ == 'TypeError'):
             pass
         else:
             raise
 
-    # Try directly using the passed value
-    try:
-        return manager.get(name_or_id, **kwargs)
-    except Exception:
-        pass
-
+    # Case 4: Try to use find.
+    # Reset the kwargs here for find
     if len(kwargs) == 0:
         kwargs = {}
 
@@ -152,7 +166,8 @@ def find_resource(manager, name_or_id, **kwargs):
         else:
             pass
 
-    # for client with no find function
+    # Case 5: For client with no find function, list all resources and hope
+    # to find a matching name or ID.
     count = 0
     for resource in manager.list():
         if (resource.get('id') == name_or_id or
@@ -216,6 +231,8 @@ def get_client_class(api_name, version, version_map):
     try:
         client_path = version_map[str(version)]
     except (KeyError, ValueError):
+        sorted_versions = sorted(version_map.keys(),
+                                 key=lambda s: list(map(int, s.split('.'))))
         msg = _(
             "Invalid %(api_name)s client version '%(version)s'. "
             "must be one of: %(version_map)s"
@@ -223,7 +240,7 @@ def get_client_class(api_name, version, version_map):
         raise exceptions.UnsupportedVersion(msg % {
             'api_name': api_name,
             'version': version,
-            'version_map': ', '.join(list(version_map.keys())),
+            'version_map': ', '.join(sorted_versions),
         })
 
     return importutils.import_class(client_path)
@@ -251,7 +268,7 @@ def get_dict_properties(item, fields, mixed_case_fields=None, formatters=None):
         else:
             field_name = field.lower().replace(' ', '_')
         data = item[field_name] if field_name in item else ''
-        if field in formatters:
+        if field in formatters and data is not None:
             row.append(formatters[field](data))
         else:
             row.append(data)
@@ -304,7 +321,7 @@ def get_item_properties(item, fields, mixed_case_fields=None, formatters=None):
         else:
             field_name = field.lower().replace(' ', '_')
         data = getattr(item, field_name, '')
-        if field in formatters:
+        if field in formatters and data is not None:
             row.append(formatters[field](data))
         else:
             row.append(data)
@@ -473,3 +490,33 @@ def wait_for_status(status_f,
             callback(progress)
         time.sleep(sleep_time)
     return retval
+
+
+def format_size(size):
+    """Display size of a resource in a human readable format
+
+    :param string size:
+        The size of the resource in bytes.
+
+    :returns:
+        Returns the size in human-friendly format
+    :rtype string:
+
+    This function converts the size (provided in bytes) of a resource
+    into a human-friendly format such as K, M, G, T, P, E, Z
+    """
+
+    suffix = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']
+    base = 1000.0
+    index = 0
+
+    if size is None:
+        size = 0
+    while size >= base:
+        index = index + 1
+        size = size / base
+
+    padded = '%.1f' % size
+    stripped = padded.rstrip('0').rstrip('.')
+
+    return '%s%s' % (stripped, suffix[index])
